function [ModelPosterior,BernoulliSummaryBayes,MarkovSummaryBayes,MLEBernoulli,MLEMarkov,LogLR,PVal,ErrorFlag]=DichoSeqTest(Sequence,NullProb,BetaPrior,delta,ComputeEstSummary,ProbContInterval)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PURPOSE: 	Bayesian and Likelihood analysis a sequence of dichotomous variables. Analyzes the model
%
%           Model 1: Independence and success prob = NullProb
%           Model 2: Independence but arbitrary success prob
%           Model 3: First order two-state Markov chain with unrestricted transition matrix:
%                              [1-pi_{01} pi_{01};
%                              1-pi_{11} pi_{11}];
%
% INPUT:	Sequence                (NSeq-by-1) Sequence of ones and zeros   
%           NullProb                (Scalar)    Bernoulli success probability under the null hypothesis
%           BetaPrior               (Scalar)    =1, uniform prior on all success and Markov transition probabilities
%                                               =2, Beta prior with mean equal to NullProb in all models. 
%           delta                   (Scalar)    If BetaPrior==2, delta is the second hyperparameter of the beta parameter which sets the precision.
%           ComputeEstSummary       (Scalar)    Computes posterior mean and posterior interval of the bernoulli and transition probs
%           ProbContInterval        (scalar)    Prob content of posterior interval
%				
% OUTPUT:	ModelPosterior          (Scalar)    Posterior probabilities of the three models listed above. 
%           BernoulliSummaryBayes   (1-by-3)    Posterior mean and posterior interval of the bernoulli success probs
%           MarkovSummaryBayes      (2-by-3)    Row gives posterior summary of pi_{01} and pi_{11}, respectively on the form:
%                                               [LowerBoundProbInterval PostMean UpperBoundProbInterval]. 
%           MLEBernoulli            (scalar) ML estimates of Bernoullu success probability
%           MLEMarkov               (1-by-2) ML estimates of pi_{01} and pi_{11}
%           LogLR                   (1-by-3) vector with Log LR test statistic (M1 against M2), (M2 against M3) and (M1 against M3)
%           PVal                    (1-by-3) P-values of the three LR tests
%
% AUTHOR:	Mattias Villani, Research Department, Sveriges Riksbank and 
%                            Department of Statistics, Stockholm University.
%                            E-mail: mattias.villani@riksbank.se.
% 
% FIRST VERSION:  2005-01-07 
% THIS VERSION:   2005-01-07
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if BetaPrior==1
    % Uniform prior on markov transition probabilties pi_{01} and pi_{11}
    alpha0=1;
    beta0=1;
    alpha01=1;
    beta01=1;
    alpha11=1;
    beta11=1;
else
    % More general prior on markov transition probabilties pi_{01} and pi_{11} which centers on the null model
    beta0=delta;
    beta01=delta;
    beta11=delta;
    alpha0=NullProb*delta/(1-NullProb);
    alpha01=NullProb*delta/(1-NullProb);
    alpha11=NullProb*delta/(1-NullProb);
end

StartZeroIndex=find(Sequence(1:end-1)==0); % Index of transitions starting in state zero.
StartUnityIndex=find(Sequence(1:end-1)==1); % Index of transitions starting in state unity.
n00=sum(Sequence(StartZeroIndex+1)==0); % Number of transitions from state 0 to state 0
n01=size(Sequence(StartZeroIndex+1),1)-n00;
n10=sum(Sequence(StartUnityIndex+1)==0);
n11=size(Sequence(StartUnityIndex+1),1)-n10;
n0=n00+n10;
n1=n01+n11;

if ComputeEstSummary==1
    
    % Bernoulli model
    alphapost0=n1+alpha0;
    betapost0=n0+beta0;
    BernoulliSummaryBayes=[betainv((1-ProbContInterval)/2,alphapost0,betapost0) alphapost0/(alphapost0+betapost0) betainv(1-(1-ProbContInterval)/2,alphapost0,betapost0)];
    
    % Markov model
    alphapost01=n01+alpha01;
    betapost01=n00+beta01;
    alphapost11=n11+alpha11;
    betapost11=n10+beta11;
    MarkovSummaryBayes(1,:)=[betainv((1-ProbContInterval)/2,alphapost01,betapost01) alphapost01/(alphapost01+betapost01) betainv(1-(1-ProbContInterval)/2,alphapost01,betapost01)];
    MarkovSummaryBayes(2,:)=[betainv((1-ProbContInterval)/2,alphapost11,betapost11) alphapost11/(alphapost11+betapost11) betainv(1-(1-ProbContInterval)/2,alphapost11,betapost11)];
else
    MarkovSummaryBayes=nan;
    BernoulliSummaryBayes=nan;
end

% MLE
warning off % Turns of the warning messages temporarily
MLEBernoulli=[n1/(n0+n1)];
if n00+n01>0 & n10+n11>0
    MLEMarkov=[n01/(n00+n01) n11/(n10+n11)];
    LogLike1=n1*log(NullProb)+n0*log(1-NullProb);
    LogLike2=n1*log(MLEBernoulli)+n0*log(1-MLEBernoulli);
    LogLike3=n01*log(MLEMarkov(1))+n00*log(1-MLEMarkov(1))+ n10*log(1-MLEMarkov(2))+n11*log(MLEMarkov(2));
    LogLR(1)=-2*(LogLike1-LogLike2);
    LogLR(2)=-2*(LogLike2-LogLike3);
    LogLR(3)=-2*(LogLike1-LogLike3);
    PVal(1)=1-chi2cdf(LogLR(1),1);
    PVal(2)=1-chi2cdf(LogLR(2),1);
    PVal(3)=1-chi2cdf(LogLR(3),2);
else
    MLEMarkov=nan;
    LogLike1=nan;
    LogLike2=nan;
    LogLike3=nan;
    LogLR=nan;
    PVal=nan;
end
warning on % Turns on the warning messages again

% Marginal likelihood and posterior probs
Logm1=n1*log(NullProb)+n0*log(1-NullProb);
Logm2=log(beta(alpha0+n1,beta0+n0))-log(beta(alpha0,beta0));
Logm3=log(beta(alpha01+n01,beta01+n00))+log(beta(alpha11+n11,beta11+n10))-log(beta(alpha01,beta01))-log(beta(alpha11,beta11));
NormConst=exp(Logm1)+exp(Logm2)+exp(Logm3);
ModelPosterior=[exp(Logm1) exp(Logm2) exp(Logm3)]/NormConst;
if n0*n1*n00*n01*n10*n11==0
    ErrorFlag=1;
    %[Logm1,Logm2,Logm3]
    %[n0,n1,n00,n01,n10,n11]
else 
    ErrorFlag=0;
end
