function [PiHat,SigmaHat,NRestrict]=MLRestrictVAR(TIMESERIES,D,k,R,Exo,QuasiML,tol,DFCorr)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PURPOSE: 	Maximum likelihood estimation of the VAR process under linear restrictions
%
% MODEL:    x_{t}=Pi_{1}*x_{t-1}+...+Pi_{K}*x_{t-K}+Phi*d_{t}+eps_{t},
%
%           where x_{t} is p-dimensional and d_{t} is q-dimensional. eps_{t} is an iid N(0,Sigma) process.
%           
%           In multivariate regression form we have: Y=Z*Phi+E,
%           where Pi=[Pi_{1},...,Pi_{k} Phi]' is (p*k+q)-by-p. 
%           The number of coefficients in the unrestricted Pi is denoted NPiPara=p^2*k+p*q;
%									
% INPUT:	TIMESERIES:     (T-by-p)    Matrix with T observations on p modelled variables.
%           D               (T-by-q)    Deterministic variables. If a constant should be included in the model, 
%                                       D should contain a column of ones.
%           k               (scalar)    Lag length of the VAR
%           R               (NPiPara,NPiPara-NRestrict) Restrictiom matrix vec(B)=R*gamma, where gamma is (NPiPara-NRestrict)-by-1 
%                                                       vector of unrestricted coefficients. If Exo is non-empty, R is overriden.
%           Exo             (NExo-by-1) Contains variable indicies for the exogenous variables. If empty, R must be supplied as user input.
%           QuasiML         (0-1)       if Quasi==1. Pi is estimated conditional on the ML estimate of Sigma. 
%                                       Fast, but only Quasi ML.
%           tol
%
% OUTPUT:	PiHat               ((p*k+q)-by-p)  ML estimate of Pi
%           SigmaHat            (p-by-p)        ML estimate of Sigma
%           NRestrict           (scalar)        The number of imposed restrictions
%
% AUTHOR:	Mattias Villani, Research Department, Sveriges Riksbank and Department of Statistics, Stockholm University.
%		    E-mail: mattias.villani@riksbank.se.
%
% REF:      Lutkepohl, H. (1991). Introduction to Multiple Time Series Analysis. Springer Verlag.
%
% FIRST VER: 2005-02-03
% THIS VER:	 2005-02-03
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin==6
    tol=10^(-5);
    DFCorr=0;
end
if nargin==7
    DFCorr=0;
end

% Constructing VAR data matrices
[T,p]=size(TIMESERIES);
[Y,X,D]=transform_to_multireg(TIMESERIES,D,k);
Z=[X D];
q=size(D,2);

% Converting to Lutkepohl notation
Y=Y';
Z=Z';

if size(Exo,1)>0
    % Constructing R from the exogeneity restriction
    NonExo=setdiff(1:p,Exo);
    NRestrict=length(NonExo)*length(Exo)*k;
    R=zeros(p^2*k+p*q,p^2*k+p*q-NRestrict);
    BStarRest=ones(p,p*k);
    for i=1:k
        BStarRest(Exo,(i-1)*p+NonExo)=0;
    end
    VecBStarRest=BStarRest(:);
    ColumnPos=0;
    for i=1:p^2*k
        if VecBStarRest(i)==1
            ColumnPos=ColumnPos+1;
            R(i,ColumnPos)=1;
        end
    end
    R(p^2*k+1:end,ColumnPos+1:ColumnPos+p*q)=eye(p*q);
else
    NRestrict=size(R,1)-size(R,2);
end

% Initial value and iteration-fixed matrices
if DFCorr==1
    Divider=(T-k*p-q);
else
    Divider=T;
end
B=Y*Z'*inv(Z*Z');
SIGMA=(1/Divider)*(Y-B*Z)*(Y-B*Z)';
z=vec(Y);
iSIGMA=inv(SIGMA);

if QuasiML==1
    % Replace SIGMA with ML estimate in the unrestricted model -> No need to iterate to obtain the MLEs.
    gammatilde=inv(R'*kron(Z*Z',iSIGMA)*R)*R'*kron(Z,iSIGMA)*z;
    betatilde=R*gammatilde;
    B=invvec(betatilde,p);
else
    % FIML. Iterative procedure. Switching between B and SIGMA.
    crit=tol+1;
    while crit>tol
        % gamma step
        gammatilde=inv(R'*kron(Z*Z',iSIGMA)*R)*R'*kron(Z,iSIGMA)*z;
        betatilde=R*gammatilde;
        Bold=B;
        B=invvec(betatilde,p);
        % SIGMA step
        SIGMA=(1/Divider)*(Y-B*Z)*(Y-B*Z)'; 
        iSIGMA=inv(SIGMA);        
        crit=norm(B-Bold);
    end
end
PiHat=B';
SigmaHat=SIGMA;