function results=identify(A)


%Returns results from tests of whether order and structure conditions are fulfilled in K-model with zero-restrictions
%USAGE:   results = identify(A)
%where:   A = nxn matrix of contemporary restrictions ("A matrix" in Amisano and Giannini's terminology)
%         A is given with ones for the free parameters, for instance
%         A=[1 0 0
%            1 1 0
%            0 1 1];
%RETURNS: results structure
%         results.ord = 1 if order condition met, 0 otherwise
%         results.str1 = 1 if structure condition met, 0 otherwise
%         results.str2 = 1 if structure condition met, 0 otherwise
%REFERENCES:    Riccardo Lucchetti, (2005), "Identification of Covariance Structures"
%               Gianni Amisano and Carlo Ginnini, (1997), "Topics in Structural VAR Econometrics"
%AUTHOR:  Pr sterholm (par.osterholm@riksbank.se or par.osterholm@nek.uu.se)
%Function modelled after article and Ox function by Lucchetti (jack@dea.unian.it)

n=size(A,1);                                    %number of variables
q=n*(n-1)/2;                                    %minimum number of restrictions needed for identification

%1. Test order condition (i.e. number of restrictions)
zeroelements=A==0;                              %nxn matrix with ones where restrictions are and zeros elsewhere
numrestr=sum(sum(zeroelements));                %number of restrictions in A
if numrestr<q
%    error('Insufficient number of restrictions provided - Order condition fails');
    h=0;
else h=1;
end

%2. Test structure condition; page 23 in Lucchetti (2005)
aaa=invvech(linspace(1,q,q)',n-1);              %generate symmetric n-1xn-1 matrix
bbb=zeros(n-1,n-1);

for i=1:n-1
    for j=1:i
        bbb(i,j)=aaa(i,j);                      %make bbb n-1xn-1 matrix with elements from aaa below diagnoal and zero otherwise
    end
end

ccc=[zeros(1,n);bbb zeros(n-1,1)];              %matrix used to generate Dtilden
Dtilden=zeros(n*n,q);

for m=1:q
    ddd=ccc==m;
    eee=ccc'==m;
    Dtilden(:,m)=vec(ddd-eee);                  %generate Dtilden by stacking column vectors
end

R=zeros(numrestr,n*n);                          %R is defined on page 8 in Lucchetti (2005)
[fff]=find(vec(zeroelements)==1);               %say which rows in vec(zeroelements) the restrictions are
for v=1:numrestr
    ggg=zeros(1,n*n);
    ggg(fff(v))=1;                              %add a 1 in the right position of the vector
    R(v,:)=ggg;                                 %generate R by stacking row vectors
end

S=zeros(n*n,n*n-numrestr);                      %S (and s) are defined on page 8 in Lucchetti (2005)
[hhh]=find(vec(zeroelements)==0);               %say which rows in vec(zeroelements) the free parameters are
for w=1:n*n-numrestr
    iii=zeros(n*n,1);
    iii(hhh(w))=1;                              %add a 1 in the right position of the vector
    S(:,w)=iii;                                 %generate S by stacking column vectors
end

Ua=[];                                          %Ua is upper left corner of T matrix page 22 in Lucchetti (2005)
for x=1:n*n-numrestr
    Ua=[Ua;R*kron(invvec(S(:,x),n)',eye(n))];   %stacking using each column in S
end
Ua=[Ua;R*kron(invvec(zeros(n*n,1),n)',eye(n))]; %stacking using the vector s (unneccesary - we only have zero restrictions)

testmatrix=Ua*Dtilden;                          %equation 29 in Lucchetti (2005)
if rank(testmatrix)<q                           %q=cols(testmatrix);
%    error('Identification fails - structure condition not fulfilled');
    z=0;
else z=1;
end

%3. Test structure condition slightly different way; page 22-23 in Lucchetti (2005)
Tmatrix=[Ua zeros(size(Ua,1),q);eye(n*n) Dtilden];%stack matrices and get T matrix
Mmatrixtest=det(Tmatrix'*Tmatrix);              %take determinant of M to determine whether it is singular
if Mmatrixtest==0
%    error('Identification fails - structure condition not fulfilled');
    c=0;
else c=1;
end

results.ord=h;                                 %returns order condition (=1 if met)
results.str1=z;                                %returns structure condition different way (=1 if met)
results.str2=c;                                %returns structure condition yet another way (=1 if met) 